"use strict";
var __extends = (this && this.__extends) || (function () {
    var extendStatics = Object.setPrototypeOf ||
        ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
        function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
var __assign = (this && this.__assign) || Object.assign || function(t) {
    for (var s, i = 1, n = arguments.length; i < n; i++) {
        s = arguments[i];
        for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
            t[p] = s[p];
    }
    return t;
};
Object.defineProperty(exports, "__esModule", { value: true });
var React = require("react");
var Form = /** @class */ (function (_super) {
    __extends(Form, _super);
    function Form(props) {
        var _this = _super.call(this, props) || this;
        _this.doSubmit = _this.doSubmit.bind(_this);
        _this.handleChange = _this.handleChange.bind(_this);
        return _this;
    }
    Form.prototype.componentWillMount = function () {
        var fields = {};
        React.Children.forEach(this.props.children, function (item) {
            var name = item.props.name;
            var value = item.props.value;
            fields[name] = value;
        });
        this.state = { fields: fields };
    };
    Form.prototype.doSubmit = function (e) {
        var _this = this;
        e.preventDefault();
        if (!this.validate())
            return;
        console.debug("Submitting values:", this.state.fields);
        this.props.onSubmit(Object.keys(this.state.fields).map(function (name) {
            return { name: name, value: _this.state.fields[name] };
        }));
    };
    Form.prototype.toArray = function (obj) {
        var array = [];
        // iterate backwards ensuring that length is an UInt32
        for (var i = obj.length >>> 0; i--;) {
            array[i] = obj[i];
        }
        return array;
    };
    Form.prototype.validate = function () {
        var _this = this;
        var validationErrors = [];
        if (this.inner_form.checkValidity && !this.inner_form.checkValidity()) {
            var inputsArray = this.toArray(this.inner_form.elements);
            inputsArray.filter(function (item) {
                return item.type !== "submit" && item.type !== "button" && !item.validity.valid;
            })
                .forEach(function (item) { return validationErrors.push({ name: item.name, validationMessage: item.validationMessage }); });
        }
        React.Children.forEach(this.props.children, function (item) {
            if (item.props && item.props.validator) {
                var customErr_1 = item.props.validator(_this.state.fields[item.props.name]);
                if (customErr_1) {
                    var exitingErr = validationErrors.filter(function (err) { return (err.name === item.props.name && err.validationMessage === customErr_1); });
                    if (!exitingErr || exitingErr.length === 0) {
                        _this.inner_form.elements[item.props.name].setCustomValidity(customErr_1);
                        validationErrors.push({ name: item.props.name, validationMessage: customErr_1 });
                    }
                }
            }
        });
        if (validationErrors.length > 0) {
            this.setState(__assign({}, this.state, { validationErrors: validationErrors }));
            return false;
        }
        this.setState(__assign({}, this.state, { validationErrors: undefined }));
        return true;
    };
    Form.prototype.handleChange = function (event) {
        var target = event.target;
        var value = target.type === "checkbox" ? target.checked : target.value;
        var name = target.name;
        var updatedFields = __assign({}, this.state.fields);
        updatedFields[name] = value;
        this.setState(__assign({}, this.state, { fields: updatedFields }));
    };
    Form.prototype.renderValidationErrors = function () {
        if (!this.state.validationErrors || this.state.validationErrors.length === 0)
            return "";
        var errs = this.state.validationErrors
            .map(function (item) { return React.createElement("li", { className: "validationErr" },
            React.createElement("span", { className: "validationErrTitle" }, item.name + ": "), item.validationMessage + "."); });
        return React.createElement("ul", { className: "validationErrList" }, errs);
    };
    Form.prototype.render = function () {
        var _this = this;
        return (React.createElement("form", { ref: function (form) { return _this.inner_form = form; }, className: "form", onSubmit: this.doSubmit, onChange: this.handleChange },
            this.props.header ? React.createElement("h1", null, this.props.header) : "",
            this.renderValidationErrors(),
            this.props.children,
            React.createElement("input", { type: "submit", onClick: this.doSubmit })));
    };
    return Form;
}(React.Component));
exports.Form = Form;
//# sourceMappingURL=Form.js.map